@@ -2,7 +2,8 @@
 # dm2seasonal: Generic function for computing seasonal values for every #
 #              year of a daily/monthly zoo object                       #
 #########################################################################
-#                  May 15th, 2009; Aug 31th 2009                        #
+# Started: 15-May-2009 ;                                                #
+# Updates: 31-Aug-2009 ; 19-Jun-2011 ; 08-Aug-2011                      #
 #########################################################################
 
 # 'x   '    : variable of type 'zoo' or 'data.frame'
@@ -16,22 +17,59 @@
 # 'na.rm'    : Logical. Should missing values be removed?
 #              TRUE : the seasonal values  are computed considering only those values different from NA
 #              FALSE: if there is AT LEAST one NA within a season, the corresponding values are NA
+# 'out.fmt'  : character indicating the format for the output time series. Possible values are:
+#              -) "%Y"      : only the year will be used for the time. Default option. (e.g., "1961" "1962"...)
+#              -) "%Y-%m-%d": a complete date format will be used for the time. Default option. (e.g., "1961" "1962"...)
+
 dm2seasonal <-function(x, ...) UseMethod("dm2seasonal")
 
 
-dm2seasonal.default <- function(x, season, FUN, na.rm=TRUE, ...) {
+dm2seasonal.default <- function(x, season, FUN, na.rm=TRUE, out.fmt="%Y", ...) {
 
-  # Checking that the user provied a valid argument for 'x'
-  if ( is.na( match( class(x), c("zoo") ) ) )
-      stop("Invalid argument: 'class(x)' must be 'zoo'")
+  # Checking that the user provied a valid class for 'x'   
+  valid.class <- c("xts", "zoo")    
+  if (length(which(!is.na(match(class(x), valid.class )))) <= 0)  
+      stop("Invalid argument: 'class(x)' must be in c('xts', 'zoo')")
+ 
+  # Requiring the Zoo Library (Z’s ordered observations)
+  require(zoo)
+
+  dm2seasonal.zoo(x=x, season=season, FUN=FUN, na.rm=na.rm, out.fmt=out.fmt, ...)
+
+} # 'dm2seasonal.default' END
+
+
+
+
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: 08-Aug-2011                 #
+# Updates: 08-Aug-2011                 #
+########################################
+dm2seasonal.zoo <- function(x, season, FUN, na.rm=TRUE, out.fmt="%Y", ...) {
 
   # Checking that the user provied a valid argument for 'season'
   if ( missing(season) ) {
       stop("Missing argument: 'season' must be provided")
-  } else # If 'season' is provided
-      # Checking a valid value for 'season'
-      if (is.na(match(season, c("DJF", "MAM", "JJA", "SON") ) ) )
-         stop("Invalid argument: 'season' must be in c('DJF', 'MAM', 'JJA', 'SON')")
+  } else { # If 'season' is provided
+    
+      seasons.default         <- c("DJF",  "MAM", "JJA",  "SON")
+      seasons.FrenchPolynesia <- c("DJFM", "AM",  "JJAS", "ON")
+          
+      # Checking that the user provied a valid class for 'season'   
+      valid.seasons <- valid.seasons <- union(seasons.default, seasons.FrenchPolynesia)
+             
+      if (length(which(!is.na(match(season, valid.seasons )))) <= 0)  
+         stop( paste("Invalid argument: 'season' must be in 'c(", paste(valid.seasons, collapse=", "), ")'",sep="") ) 
+            
+      # Finding out if 'season' belongs to 'seasons.default' or to ' seasons.FrenchPolynesia'.
+      if ( season %in% seasons.default ) {
+        season.type <- "default"
+      } else if ( season %in% seasons.FrenchPolynesia ) {
+          season.type <- "FrenchPolynesia"
+        } # ELSE end
+            
+    } # ELSE end
 
   # Checking that the user provied a valid argument for 'FUN'
   if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")
@@ -39,65 +77,59 @@ dm2seasonal.default <- function(x, season, FUN, na.rm=TRUE, ...) {
   # Checking that 'x' is a Daily or Monthly ts
   if (is.na(match(sfreq(x), c("daily", "monthly") ) ) )
       stop(paste("Invalid argument: 'x' must be a daily or monthly ts, but 'sfreq(x)' is", sfreq(x), sep=" ")  )
-
-  dates <- time(x)
-
-  # Computing the Starting and Ending Year of the analysis
-  Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
-  Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
-
-  # Amount of Years belonging to the desired season
-  nyears <- Ending.Year - Starting.Year + 1
-
-  # Requiring the Zoo Library
-  require(zoo)
-
-  # If 'x' is a daily ts, the Monthly values are computed
-  #if ( sfreq(x) == "Daily")  {
-  #   x <- daily2monthly(x, FUN, na.rm ) }
+      
+  # Checking 'out.fmt'
+  if ( is.na(match(out.fmt, c("%Y", "%Y-%m-%d") ) ) )
+    stop("Invalid argument: 'out.fmt' must be in c('%Y', '%Y-%m-%d')" )
 
   # Getting the Monthly values beloonging ONLY to the desired weather season
-  s <- extractzoo(x= x, trgt=season)
+  s <- extract(x= x, trgt=season)
 
   # Moving forward all the December values, in order that
   # December of 1991 be used together with Jan/92 and Feb/92,
   # instead of with Jan/91 and Feb/91
-  if (season == "DJF") {
-			syears            <- as.numeric(format( time(s), "%Y" ))
-			dec.index         <- which(format(time(s), "%m") == 12)
-			dec.years         <- syears[dec.index]
-			dec.years         <- dec.years + 1
-			syears[dec.index] <- dec.years
+  if ( (season == "DJF") | (season == "DJFM") ) {
+  
+	syears            <- as.numeric(format( time(s), "%Y" ))
+	dec.index         <- which(format(time(s), "%m") == 12)
+	dec.years         <- syears[dec.index]
+	dec.years         <- dec.years + 1
+	syears[dec.index] <- dec.years
 
-			s.a <- aggregate( s, by= syears, FUN=FUN, na.rm= na.rm)
+	s.a <- aggregate( s, by= syears, FUN=FUN, na.rm= na.rm)
 
-			# Removing the last value of december, because it is outside of the analysis period
-			s.a <- s.a[1:(length(s.a)-1)]
-  } else  {
+	# Removing the last value of december, because it is outside of the analysis period
+	if ( (is.matrix(x)) | (is.data.frame(x)) ) {
+	  s.a <- s.a[1:(nrow(s.a)-1), ]
+	} else s.a <- s.a[1:(length(s.a)-1)]
+			
+  } else  s.a <- aggregate( s, by= format( time(s), "%Y" ), FUN=FUN, na.rm= na.rm )	
 
-			s.a <- aggregate( s, by= format( time(s), "%Y" ), FUN=FUN, na.rm= na.rm )
-	}
-
-  # Getting the position of all the years in which there were no values
+  # Replacing the NaNs by 'NA.
   # mean(NA:NA, na.rm=TRUE) == NaN
   nan.index <- which(is.nan(s.a))
+  if ( length(nan.index) > 0 ) s.a[nan.index] <- NA 
 
-  # Changing all the NaN's by NA's
-  if ( length(nan.index) > 0 ) { s.a[nan.index] <- NA }
-
-  # Getting the position of all the years in which there were no values
+  # Replacing all the Inf and -Inf by NA's
   # min(NA:NA, na.rm=TRUE) == Inf  ; max(NA:NA, na.rm=TRUE) == -Inf
   inf.index <- which(is.infinite(s.a))
-
-  # Changing all the Inf and -Inf by NA's
-  if ( length(inf.index) > 0 ) { s.a[inf.index] <- NA }
+  if ( length(inf.index) > 0 ) s.a[inf.index] <- NA 
+  
+  # If the user wants a complete data format for the output ts:
+  if (out.fmt == "%Y-%m-%d")
+    time(s.a) <- as.Date(paste( time(s.a), "-01-01", sep=""))
 
   return(s.a)
 
-} # 'dm2seasonal.default' END
+} # 'dm2seasonal.zoo' END
 
 
 
+########################################
+# Author : Mauricio Zambrano-Bigiarini #
+# Started: 15-May-2009                 #
+# Updates: 08-Aug-2011                 #
+########################################
 # 'dates'   : "numeric", "factor", "Date" indicating how to obtain the
 #             dates for correponding to the 'sname' station
 #             If 'dates' is a number, it indicates the index of the column in
@@ -119,7 +151,8 @@ dm2seasonal.default <- function(x, season, FUN, na.rm=TRUE, ...) {
 #                              The fouth column will contain the seasonal value, corresponding to the year specified in the second column
 dm2seasonal.data.frame <- function(x, season, FUN, na.rm=TRUE,
                                    dates, date.fmt="%Y-%m-%d",
-				   out.type="data.frame",... ) {
+				   out.type="data.frame", 
+				   out.fmt="%Y", ... ) {
 
   # Checking that the user provied a valid argument for 'out.type'
   if (is.na(match( out.type, c("data.frame", "db") ) ) )
@@ -128,13 +161,32 @@ dm2seasonal.data.frame <- function(x, season, FUN, na.rm=TRUE,
   # Checking that the user provied a valid argument for 'season'
   if ( missing(season) ) {
       stop("Missing argument: 'season' must be provided")
-  } else # If 'season' is provided
-      # Checking a valid value for 'season'
-      if (is.na(match(season, c("DJF", "MAM", "JJA", "SON") ) ) )
-         stop("Invalid argument: 'season' must be in c('DJF', 'MAM', 'JJA', 'SON')")
+  } else { # If 'season' is provided
+    
+      seasons.default         <- c("DJF",  "MAM", "JJA",  "SON")
+      seasons.FrenchPolynesia <- c("DJFM", "AM",  "JJAS", "ON")
+          
+      # Checking that the user provied a valid class for 'season'   
+      valid.seasons <- valid.seasons <- union(seasons.default, seasons.FrenchPolynesia)
+             
+      if (length(which(!is.na(match(season, valid.seasons )))) <= 0)  
+         stop( paste("Invalid argument: 'season' must be in 'c(", paste(valid.seasons, collapse=", "), ")'",sep="") ) 
+            
+      # Finding out if 'season' belongs to 'seasons.default' or to ' seasons.FrenchPolynesia'.
+      if ( season %in% seasons.default ) {
+        season.type <- "default"
+      } else if ( season %in% seasons.FrenchPolynesia ) {
+          season.type <- "FrenchPolynesia"
+        } # ELSE end
+            
+    } # ELSE end
 
   # Checking that the user provied a valid argument for 'FUN'
   if (missing(FUN)) stop("Missing argument: 'FUN' must be provided")
+  
+  # Checking 'out.fmt'
+  if ( is.na(match(out.fmt, c("%Y", "%Y-%m-%d") ) ) )
+    stop("Invalid argument: 'out.fmt' must be in c('%Y', '%Y-%m-%d')" )
 
   # Checking that the user provied a valid argument for 'dates'
   if (missing(dates)) {
@@ -162,58 +214,31 @@ dm2seasonal.data.frame <- function(x, season, FUN, na.rm=TRUE,
   if ( ( class(dates) == "Date") & (length(dates) != nrow(x) ) )
      stop("Invalid argument: 'length(dates)' must be equal to 'nrow(x)'")
 
-  # Checking a valid value for 'season'
-  if (is.na(match(season, c("DJF", "MAM", "JJA", "SON") ) ) )
-     stop("Invalid argument: 'season' must be in c('DJF', 'MAM', 'JJA', 'SON')")
-
-  # Amount of stations in 'x'
-  nstations <- ncol(x)
-
-  # ID of all the stations in 'x'
-  snames <- colnames(x)
-
-  # Computing the Starting and Ending Year of the analysis
-  Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
-  Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
-
-  # Amount of Years belonging to the desired season
-  nyears <- Ending.Year - Starting.Year + 1
-
-  # Requiring the Zoo Library (Z’s ordered observations)
-  require(zoo)
-
-  message("[Starting the computations...]")
-
+  # Transforming 'x' into zoo
+  x <- zoo(x, dates)  
+     
+  ##############################################################################
   if (out.type == "data.frame") {
-
-	# Vector with the names of the field that will be used for storing the results
-	field.names <- c(snames )
-
-	# Creating the data.frame that will store the computed averages for each station
-	z <- as.data.frame(matrix(data = NA, nrow = nyears, ncol = nstations,
-                                  byrow = TRUE, dimnames = NULL) )
-	colnames(z) <- field.names
-
-	rownames(z) <- Starting.Year:Ending.Year
-
-	z[1:nstations] <- sapply(1:nstations, function(j,y) {
-
-		message( paste("Station: ", format(snames[j], width=10, justify="left"),
-                               " : ",format(j, width=3, justify="left"), "/",
-                               nstations, " => ",
-                               format(round(100*j/nstations,2), width=6, justify="left"),
-                               "%", sep="") )
-
-		# Transforming the column of 'x' into a zoo object,
-		# using the dates provided by the user
-		tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)
-
-		z[,j] <- dm2seasonal.default(x= tmp, season=season, FUN=FUN, na.rm=na.rm)
-
-	}, y = x) # sapply END
-
+  
+    dm2seasonal.zoo(x=x, season=season, FUN=FUN, na.rm=na.rm, out.fmt=out.fmt, ...)
+    
   } else if (out.type == "db") {
 
+        # Amount of stations in 'x'
+        nstations <- ncol(x)
+
+        # ID of all the stations in 'x'
+        snames <- colnames(x)
+
+        # Computing the Starting and Ending Year of the analysis
+        Starting.Year <- as.numeric(format(range(dates)[1], "%Y"))
+        Ending.Year   <- as.numeric(format(range(dates)[2], "%Y"))
+
+        # Amount of Years belonging to the desired season
+        nyears <- Ending.Year - Starting.Year + 1
+
+        message("[Starting the computations...]")
+
         # Creating a vector with the names of the field that will be used for storing the results
         field.names <- c("StationID", "Year", "Season", "Value" )
 
@@ -226,18 +251,18 @@ dm2seasonal.data.frame <- function(x, season, FUN, na.rm=TRUE,
 
         for (j in 1:nstations) {
 
-            message( paste("Station: ", format(snames[j], width=10, justify="left"),
+            message( paste("[ Station: ", format(snames[j], width=10, justify="left"),
                          " : ",format(j, width=3, justify="left"), "/",
                          nstations, " => ",
                          format(round(100*j/nstations,2), width=6, justify="left"),
-                         "%", sep="") )
+                         "% ]", sep="") )
 
             # Transforming the column of 'x' into a zoo object,
 	    # using the dates provided by the user
             tmp <- vector2zoo(x=x[,j], dates=dates, date.fmt=date.fmt)
 
 	    # Computing the seasonal values
-            s.a <- dm2seasonal.default(x= tmp, season=season, FUN=FUN, na.rm=na.rm)
+            s.a <- dm2seasonal.default(x= tmp, season=season, FUN=FUN, na.rm=na.rm, out.fmt=out.fmt)
 
             # Putting the annual seasonal values in the output data.frame
             # The first column of 'x' corresponds to the Year
@@ -250,9 +275,24 @@ dm2seasonal.data.frame <- function(x, season, FUN, na.rm=TRUE,
             z[row.ini:row.fin, 4] <- s.a
 
         } # FOR end
+        
+        return( z )
 
     } # IF end
 
-  return( z )
-
  } #'dm2seasonal.data.frame' END
+ 
+ 
+dm2seasonal.matrix  <- function(x, season, FUN, na.rm=TRUE,
+                                dates, date.fmt="%Y-%m-%d",
+				out.type="data.frame", 
+				out.fmt="%Y", ... ) {
+
+   x <- as.data.frame(x)
+   #NextMethod("daily2annual")
+   dm2seasonal.data.frame(x=x, season=season, FUN=FUN, na.rm=na.rm,
+                          dates=dates, date.fmt=date.fmt,
+			  out.type=out.type, 
+			  out.fmt=out.fmt, ... )
+
+} # 'dm2seasonal.matrix  ' END
